/************************Copyright (c)*******************************************************************
*********************************************************************************************************
**                 合肥途思电子科技有限公司            
**                        研发中心                             
**                         硬件部                                
**                                                               
**                http://www.tinysine.com                                 
**
**-------------文件信息---------------------------------------------------------------------------------
****文   件   名：GPS

****版        本：V1r1
****日        期：2020年2月21日
****程   序   员：HuangChao
****描        述：for 4G Shield V1r1
         
****版        本：V1r0
****日        期：2020年2月4日
****程   序   员：HuangChao
****描        述： 
****备        注: 
                1.Please use Arduino 1.0.1 version
                2.Please select the Mega2560 board or UNO
**              
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/

/********************************************************************************************************
 * 										INCLUDE FILES
********************************************************************************************************/
#include <SoftwareSerial.h>

/********************************************************************************************************
 *CONSTANTS
********************************************************************************************************/
#define CR                   0x0D                    // carriage return
#define LF                   0x0A                    // line feed

#define SUB                  0x1A                    // substitute ctrl-Z

#define GREATER_THAN_SIGN    0x3E                    // >
#define SPACE                0x20


/********************************************************************************************************
 * 										VARIABLES DEFINITION
********************************************************************************************************/
HardwareSerial &SerialDebug=Serial;

SoftwareSerial SerialSim7600(2, 3);                 //for Uno,the software serial port rx pin is D2, tx pin is D3
//HardwareSerial &SerialSim7600=Serial2;            //for Mega2560,the hardware serial port is the serial2

boolean SIM7600ReadyFlag=false;
char StringOK[]="OK";
char StringGreaterThanSign[]="> ";

/********************************************************************************************************
 * 										FUNCTION PROTOTYPES
********************************************************************************************************/

/*********************************************************************************************************
** 函数名称: void setup()
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void setup()
{
  unsigned char i=0;
  
  SerialDebug.begin(9600);  
  
  SerialSim7600.begin(9600); 
  SerialSim7600.setTimeout(1000);  //1s
  
  for(i=0;i<3;i++)
  {
    SerialDebug.print(F("Try AT test "));
    SerialDebug.println(i);
    
    if(true==ATTest())
    {
      SIM7600ReadyFlag=true;
      break;  
    }
    delay(1000);
    SerialSim7600FlushRxBuf();
  }  
  
  GPSInit();
}

/*********************************************************************************************************
** 函数名称: void loop()
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void loop()
{
  if(true==SIM7600ReadyFlag)
  {
    GPS();
  }
}

/*********************************************************************************************************
** 函数名称: void GPSInit(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void GPSInit(void)
{
  SerialSim7600.print(F("AT+CGPS=1"));
  SerialSim7600.write(CR); 
  
  SerialSim7600.flush(); 
  
  if(SerialSim7600.find(StringOK))
  {
    SerialDebug.println(F("Start GPS"));
  }
}

/*********************************************************************************************************
** 函数名称: void GPS(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void GPS(void)
{
  char keypad_value; 
  boolean ok_result=false;
  unsigned int counter=500;

  SerialDebugFlushRxBuf();
  SerialSim7600FlushRxBuf();
    
  SerialDebug.println();
  SerialDebug.println(F("****************************************"));
  SerialDebug.println(F("Get GPS Position"));
  SerialDebug.println(F("Do you want to get position?"));
  SerialDebug.println(F("Please type y"));
  
  while(!SerialDebug.available());
  
  keypad_value=SerialDebug.read();
  if(('Y'==keypad_value)||('y'==keypad_value))
  {
    while(1)
    {
      SerialDebug.println();
      SerialDebug.println(F("****************************************"));
      SerialDebug.println(F("Reading position...")); 
    
      SerialSim7600.print(F("AT+CGPSINFO"));
      SerialSim7600.write(CR);    
      SerialSim7600.flush(); 
      for(counter=500;counter>0;counter--)
      {
        delay(10);
        while((SerialSim7600.available()>0))
        {
          SerialDebug.write(SerialSim7600.read());
        }
      }
    }
  }   
}

/*********************************************************************************************************
** 函数名称: void SerialSim7600FlushRxBuf(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void SerialSim7600FlushRxBuf(void)
{
  while(SerialSim7600.available())
  {
    SerialSim7600.read();
  }
}

/*********************************************************************************************************
** 函数名称: void SerialDebugFlushRxBuf(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void SerialDebugFlushRxBuf(void)
{
  while(SerialDebug.available())
  {
    SerialDebug.read();
  }
}

/*********************************************************************************************************
** 函数名称: boolean ATTest(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: boolean      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
boolean ATTest(void)
{
  boolean at_test_result=false;
  
  SerialSim7600.print(F("AT"));
  SerialSim7600.write(CR); 
  
  SerialSim7600.flush();                          //Waits for the transmission of outgoing serial data to complete
  
  at_test_result=SerialSim7600.find(StringOK);    
  
  if(true==at_test_result)
  {
    SerialDebug.println(F("ATtest OK"));
  }
  else
  {
    SerialDebug.println(F("ATtest Fail"));
  }
  return(at_test_result);
}

/********************************************************************************************************
**                            End Of File
********************************************************************************************************/ 




