/************************Copyright (c)*******************************************************************
*********************************************************************************************************
**                 合肥途思电子科技有限公司            
**                        研发中心                             
**                         硬件部                                
**                                                               
**                http://www.tinysine.com                                 
**
**-------------文件信息---------------------------------------------------------------------------------
****文   件   名：PowerOnOff2

****版        本：V1r1
****日        期：2020年2月21日
****程   序   员：HuangChao
****描        述：for 4G Shield V1r1
         
****版        本：V1r0
****日        期：2020年2月20日
****程   序   员：HuangChao
****描        述：Use the status pin to judge the module status
****备        注: 
                1.Please use Arduino 1.0.1 version
                2.Please select the Mega2560 board or UNO
**              
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/

/********************************************************************************************************
 * 										INCLUDE FILES
********************************************************************************************************/
#include <SoftwareSerial.h>

/********************************************************************************************************
 *CONSTANTS
********************************************************************************************************/
#define CR                   0x0D                    // carriage return
#define LF                   0x0A                    // line feed

#define SUB                  0x1A                    // substitute ctrl-Z

#define GREATER_THAN_SIGN    0x3E                    // >
#define SPACE                0x20

#define POWER_PIN            8
#define ANALOG_PIN           A0

#define VOLTAGE_ON_TH        1.0                    //the status pin voltage ON:1.8V,OFF:0V . We set the threshold voltage 1.0V


/********************************************************************************************************
 * 										VARIABLES DEFINITION
********************************************************************************************************/
HardwareSerial &SerialDebug=Serial;

SoftwareSerial SerialSim7600(2, 3);                 //for Uno,the software serial port rx pin is D2, tx pin is D3
//HardwareSerial &SerialSim7600=Serial2;            //for Mega2560,the hardware serial port is the serial2

boolean SIM7600_On_Flag=false;
char StringOK[]="OK";
char StringNOCARRIER[]="NO CARRIER";
char StringRING[]="RING";

float VoltageStatusPin=0;



/********************************************************************************************************
 * 										FUNCTION PROTOTYPES
********************************************************************************************************/

/*********************************************************************************************************
** 函数名称: void AnalogInit(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void AnalogInit(void)
{
  analogReference(DEFAULT); 
}

/*********************************************************************************************************
** 函数名称: void AnalogProc(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
float AnalogProc(void)
{
  unsigned int analog_value[3];
  unsigned int analog_average_value=0;
  float  voltage=0;
  unsigned i=0;
  for(i=0;i<3;i++)
  {
    analog_value[i] = analogRead(ANALOG_PIN);   //about   100 microseconds
    delay(100);                                  //100ms
    analog_average_value+=analog_value[i];
    //SerialDebug.println(analog_value[i]);
  }
  analog_average_value=analog_average_value/3;
  //SerialDebug.println(analog_average_value);
  voltage=((float)analog_average_value)*5 /1024;
  //SerialDebug.println(voltage);
  return (voltage);
}


/*********************************************************************************************************
** 函数名称: void PowerPinInit(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void PowerPinInit(void)
{
  pinMode(POWER_PIN, OUTPUT); 
  digitalWrite(POWER_PIN, LOW);
}

/*********************************************************************************************************
** 函数名称: void setup()
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void setup()
{
  unsigned char i=0;
  
  AnalogInit();
  
  PowerPinInit();
  
  SerialDebug.begin(9600);  
  
  SerialSim7600.begin(9600); 
  SerialSim7600.setTimeout(1000);  //1s
  
  VoltageStatusPin=AnalogProc();
  
  if(VoltageStatusPin<VOLTAGE_ON_TH)
  {
    SIM7600_On_Flag=false;
  }
  else
  {
    SIM7600_On_Flag=true;
  }
}

/*********************************************************************************************************
** 函数名称: void loop()
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void loop()
{
  SerialDebug.println();
  SerialDebug.print(F("The status pin voltage is "));
  SerialDebug.print(VoltageStatusPin);
  SerialDebug.println(F("V.")); 
  if(false==SIM7600_On_Flag)
  {
    PowerOn();
  }
  else
  {
    PowerOff();
  }
}

/*********************************************************************************************************
** 函数名称: void PowerOn(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void PowerOn(void)
{
  char keypad_value; 
  unsigned char i;
  
  SerialSim7600FlushRxBuf();
  SerialDebugFlushRxBuf();
   
  SerialDebug.println(F("The module is power off"));
  SerialDebug.println(F("****************************************"));
  SerialDebug.println(F("POWER ON Sequence"));
  SerialDebug.println(F("Do you want to power on the SIM7600?"));
  SerialDebug.println(F("Please type y"));
  
  while(!SerialDebug.available());
  
  keypad_value=SerialDebug.read();
  if(('Y'==keypad_value)||('y'==keypad_value))
  {
    SerialDebug.println(F("1.Pull down the powerkey"));
    digitalWrite(POWER_PIN, HIGH);
    delay(1000);
    digitalWrite(POWER_PIN, LOW);
    
    SerialDebug.println(F("2.Waitting,May spend tens of seconds"));
    delay(16000);
    SerialSim7600FlushRxBuf();
    SerialDebugFlushRxBuf();

    for(i=0;i<10;i++)
    {
      VoltageStatusPin=AnalogProc();
      if(VoltageStatusPin>=VOLTAGE_ON_TH)
      {
        SIM7600_On_Flag=true;
        break;
      }
      delay(1000);
    }
    if(false==SIM7600_On_Flag)
    {
      SerialDebug.println(F("Fail,Please check the hardware"));
      while(1);
    }
    else
    {
      SerialDebug.println(F("Success!"));
    }
  }
  else
  {
    SerialDebug.println(F("invalid char"));
    delay(1000);
  }
}

/*********************************************************************************************************
** 函数名称: void PowerOff(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void PowerOff(void)
{
  char keypad_value; 
  unsigned char i;

  SerialSim7600FlushRxBuf();
  SerialDebugFlushRxBuf();
  
  SerialDebug.println(F("The module is power on"));
  SerialDebug.println(F("****************************************"));
  SerialDebug.println(F("POWER OFF Sequence"));
  SerialDebug.println(F("Do you want to power off the SIM7600?"));
  SerialDebug.println(F("Please type y"));
  
  while(!SerialDebug.available());
  
  keypad_value=SerialDebug.read();
  if(('Y'==keypad_value)||('y'==keypad_value))
  {
    SerialDebug.println(F("1.Waitting,May spend tens of seconds."));
    
    SerialSim7600.print("AT+CPOF");
    SerialSim7600.write(CR);    
    SerialSim7600.flush();
    SerialSim7600.find(StringOK); 
    
    delay(26000);                                      //26s
    SerialSim7600FlushRxBuf();
    SerialDebugFlushRxBuf();
    
    for(i=0;i<10;i++)
    {
      VoltageStatusPin=AnalogProc();
      if(VoltageStatusPin<VOLTAGE_ON_TH)
      {
        SIM7600_On_Flag=false;
        break;
      }
      delay(1000);
    }
    if(true==SIM7600_On_Flag)
    {
      SerialDebug.println(F("Fail,Please check the hardware"));
      while(1);
    }
    else
    {
      SerialDebug.println(F("Success!"));
    }
  }
  else
  {
    SerialDebug.println(F("invalid char"));
    delay(1000);
  }
}

/*********************************************************************************************************
** 函数名称: void SerialSim7600FlushRxBuf(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void SerialSim7600FlushRxBuf(void)
{
  while(SerialSim7600.available())
  {
    SerialSim7600.read();
  }
}

/*********************************************************************************************************
** 函数名称: void SerialDebugFlushRxBuf(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: void      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
void SerialDebugFlushRxBuf(void)
{
  while(SerialDebug.available())
  {
    SerialDebug.read();
  }
}

/*********************************************************************************************************
** 函数名称: boolean ATTest(void)
** 功能描述: 
** 输　  入: void 
** 输　  出: boolean      
**-------------------------------------------------------------------------------------------------------
** 说    明:	
** 日　  期:
**------------------------------------------------------------------------------------------------------
********************************************************************************************************/
boolean ATTest(void)
{
  boolean at_test_result=false;
  
  SerialSim7600.print(F("AT"));
  SerialSim7600.write(CR); 
  SerialSim7600.flush();                          //Waits for the transmission of outgoing serial data to complete
  
  at_test_result=SerialSim7600.find(StringOK);    
  
  if(true==at_test_result)
  {
    SerialDebug.println(F("ATtest OK"));
  }
  else
  {
    SerialDebug.println(F("ATtest Fail"));
  }
  return(at_test_result);
}


/********************************************************************************************************
**                            End Of File
********************************************************************************************************/ 




